@@ -25,7 +25,10 @@
 
 #include <config.h>
 #include "gst.h"
+
+#ifdef HAVE_POLKIT
 #include "gst-polkit-action.h"
+#endif
 
 #include "passwd.h"
 #include "callbacks.h"
@@ -265,13 +268,12 @@ on_user_new_clicked (GtkButton *button, gpointer user_data)
 	OobsListIter list_iter;
 	gint response;
 
-	user = oobs_user_new (NULL);
-	dialog = user_settings_dialog_new (user);
+	dialog = user_settings_dialog_new (NULL);
 	gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (tool->main_dialog));
-	response = user_settings_dialog_run (dialog, user);
+	response = user_settings_dialog_run (dialog);
 
 	if (response == GTK_RESPONSE_OK) {
-		user_settings_dialog_get_data (user);
+		user = user_settings_dialog_get_data (dialog);
 
 		config = OOBS_USERS_CONFIG (GST_USERS_TOOL (tool)->users_config);
 		users_list = oobs_users_config_get_users (config);
@@ -311,12 +313,13 @@ on_user_settings_clicked (GtkButton *button, gpointer user_data)
 			    -1);
 	dialog = user_settings_dialog_new (user);
 	gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (tool->main_dialog));
-	response = user_settings_dialog_run (dialog, user);
+	response = user_settings_dialog_run (dialog);
+	g_object_unref (user);
 
 	if (response == GTK_RESPONSE_OK) {
 		gtk_tree_model_filter_convert_iter_to_child_iter (GTK_TREE_MODEL_FILTER (model),
 								  &filter_iter, &iter);
-		user_settings_dialog_get_data (user);
+		user = user_settings_dialog_get_data (dialog);
 		users_table_set_user (user, list_iter, &filter_iter);
 
 		if (gst_dialog_is_authenticated (tool->main_dialog)) {
@@ -328,17 +331,20 @@ on_user_settings_clicked (GtkButton *button, gpointer user_data)
 
 			/* change self, only if it is the modified user */
 			if (user == oobs_self_config_get_user (OOBS_SELF_CONFIG (object))) {
+#ifdef HAVE_POLKIT
 				GstPolKitAction *action;
 				action = gst_polkit_action_new (oobs_object_get_authentication_action (object),
 								GTK_WIDGET (tool->main_dialog));
 
 				if (gst_polkit_action_authenticate (action))
 					oobs_object_commit (GST_USERS_TOOL (tool)->self_config);
+
+				g_object_unref (action);
+#endif
 			}
 		}
 	}
 
-	g_object_unref (user);
 	oobs_list_iter_free (list_iter);
 }
 
@@ -532,23 +538,20 @@ on_user_settings_login_changed (GtkEditable *editable,
 	gchar *home, *base_dir;
 	GtkWidget *home_entry;
 
-	if (user_settings_dialog_user_is_new ()) {
-		home_entry = gst_dialog_get_widget (tool->main_dialog, "user_settings_home");
+	home_entry = gst_dialog_get_widget (tool->main_dialog, "user_settings_home");
+	base_dir = g_object_get_data (G_OBJECT (home_entry), "default-home");
 
-		base_dir = g_object_get_data (G_OBJECT (home_entry), "default-home");
+	if (!base_dir)
+		g_object_get (GST_USERS_TOOL (tool)->users_config,
+			      "default-home", &base_dir, NULL);
 
-		if (!base_dir)
-			g_object_get (GST_USERS_TOOL (tool)->users_config,
-				      "default-home", &base_dir, NULL);
+	home = g_build_path (G_DIR_SEPARATOR_S,
+			     base_dir,
+			     gtk_entry_get_text (GTK_ENTRY (editable)),
+			     NULL);
 
-		home = g_build_path (G_DIR_SEPARATOR_S,
-				     base_dir,
-				     gtk_entry_get_text (GTK_ENTRY (editable)),
-				     NULL);
-
-		gtk_entry_set_text (GTK_ENTRY (home_entry), home);
-		g_free (home);
-	}
+	gtk_entry_set_text (GTK_ENTRY (home_entry), home);
+	g_free (home);
 }
 
 void
